<?php

namespace Webkul\SalesforceConnector\Block\Adminhtml\System\Config;

use Magento\Backend\Block\Template\Context;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Catalog\Ui\DataProvider\Product\Attributes\Listing;
use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\Context as HelperContext;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Store\Model\ScopeInterface;
use Throwable;
use Webkul\SalesforceConnector\Helper\Connection;
use Webkul\SalesforceConnector\Helper\Data;

/**
 * Class ProdMap
 */
class ProdMap extends Field
{
    /**
     * @var Connection
     */
    protected $helper;

    /**
     * @var Json
     */
    protected $json;

    /**
     * @var Data
     */
    protected $helperData;

    /**
     * @var CollectionFactory
     */
    protected $productCollectionFactory;

    /**
     * @var Listing
     */
    protected $dataProvider;

    /**
     * @var RequestInterface
     */
    protected $request;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopConfig;

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * ProdMap constructor.
     *
     * @param Context $context
     * @param Connection $helper
     * @param HelperContext $helperContext
     * @param Data $helperData
     * @param Json $jsonHelper
     * @param CollectionFactory $productCollectionFactory
     * @param RequestInterface $request
     * @param array $data
     */
    public function __construct(
        Context $context,
        Connection $helper,
        ScopeConfigInterface $scopeConfig,
        Data $helperData,
        Json $jsonHelper,
        CollectionFactory $productCollectionFactory,
        RequestInterface $request,
        ResourceConnection $resourceConnection,
        array $data = []
    ) {
        parent::__construct($context, $data);

        $this->helper = $helper;
        $this->json = $jsonHelper;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->helperData = $helperData;
        $this->scopConfig = $scopeConfig;
        $this->request = $request;
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return $this|ProdMap
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();

        if (!$this->getTemplate()) {
            $this->setTemplate(
                'Webkul_SalesforceConnector::system/config/prodfieldmap.phtml'
            );
        }
        return $this;
    }

    /**
     * Unset some non-related element parameters
     *
     * @param AbstractElement $element
     * @return string
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()
            ->unsCanUseWebsiteValue()
            ->unsCanUseDefaultValue();

        return parent::render($element);
    }

    /**
     * Get the button and scripts contents
     *
     * @param AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * @return int
     */
    public function getProductObject()
    {
        $account = $this->scopConfig
            ->getValue(
                'sfConnect/general/productObject',
                ScopeInterface::SCOPE_STORE
            );
        if ($account) {
            return $this->json->unserialize($account);
        } else {
            return 0;
        }
    }

    /**
     * @return void
     */
    protected function setUp()
    {
        $objectManager = ObjectManager::getInstance();

        $this->dataProvider = $objectManager->create(
            Listing::class,
            [
                'name' => 'product_attributes_grid_data_source',
                'primaryFieldName' => 'attribute_id',
                'requestFieldName' => 'id',
                'request' => $this->request
            ]
        );
    }

    /**
     * @return array
     */
    public function getProductTable()
    {
        $this->setUp();
        $this->dataProvider->addOrder('attribute_code', 'asc');
        $data = $this->dataProvider->getData();

        $items = $data['items'];
        foreach ($this->getInventoryFields() as $field) {
            $items[] = $field;
        }

        return $items;
    }

    /**
     * @return array
     */
    protected function getInventoryFields()
    {
        try {
            $connection = $this->resourceConnection->getConnection();

            $query = sprintf(
                'SHOW FULL COLUMNS FROM `%s`',
                $connection->getTableName('cataloginventory_stock_item')
            );

            $fields = [];
            foreach ($connection->query($query)->fetchAll() as $field) {
                if (!preg_match('/decimal/i', $field['Type']) && $field['Field'] != 'is_in_stock') {
                    continue;
                }

                $fields[] = [
                    'attribute_code' => $field['Field'],
                    'frontend_input' => str_replace(' unsigned', '', $field['Type']),
                    'frontend_label' => '(Inventory) ' . $field['Field']
                ];
            }

            return $fields;
        } catch (Throwable $e) {
            return [];
        }
    }

    /**
     * @return int|mixed
     */
    public function getMappedProduct()
    {
        $account = $this->scopConfig
            ->getValue(
                'sfConnect/acc/productFieldMap',
                ScopeInterface::SCOPE_STORE
            );
        if ($account) {
            return $this->json->unserialize($account);
        } else {
            return 0;
        }
    }

    /**
     * @return Collection
     */
    public function getProductCollection()
    {
        $collection = $this->productCollectionFactory->create();
        $collection->addAttributeToSelect('*');

        return $collection;
    }
}
